#pragma once

#include "TranslateFileInfo.h"

#define DEF_CHAT_MSG_MIN_FONT_SIZE		(8)
#define MAX_USER_COUNT 1024

typedef enum _chat_msg_type
{
  CHAT_MSG_NONE = 0,
  CHAT_MSG_SYS,
  CHAT_MSG_CHAT,
  CHAT_MSG_FTP,
  CHAT_MSG_STICKER,
  CHAT_MSG_JOIN,
  CHAT_MSG_LEAVE,
  CHAT_MSG_RECOVER_SESSION

}ECHAT_MSG_TYPE;

typedef enum ChatMsgStatus
{
	CMS_INVALID = -1,
	CMS_SUCCEED,
	CMS_FAIL

}CHAT_MSG_STATUS,*LPCHAT_MSG_STATUS;



typedef struct stu_ChatItemInfo : public CObject
{
	DECLARE_SERIAL(stu_ChatItemInfo)

	ECHAT_MSG_TYPE	    eMsgGB;	
	CString				strUserID;
	CString				strNameTalkName;
	CString				strUserName;
	CString				strMsg;
	CHARFORMAT2			cf;
	CTime               time;
	UINT                nTid;
	UINT                nMsgSeq;
	int					nUserCount;
	int					nMsgStatus;
	int					m_nOsType;

	stu_ChatItemInfo()
	{
		eMsgGB = CHAT_MSG_NONE;	
		strUserID = _T("");
		strNameTalkName = _T("");
		strUserName = _T("");
		strMsg = _T("");
		ZeroMemory(&cf, sizeof(cf));  //lint !e1415
		nTid = 0;
		nMsgSeq =0;
		nUserCount = 0;
		nMsgStatus = CMS_INVALID;
		m_nOsType = 0;
	}

	~stu_ChatItemInfo()
	{
	}

	stu_ChatItemInfo(LPCTSTR szMsg, CTime chattime, CHARFORMAT2 cformat, ECHAT_MSG_TYPE emMsgType = CHAT_MSG_SYS,int userCount = 0)
	{
		eMsgGB = emMsgType;	
		strUserID = _T("");
		strNameTalkName = _T("");
		strUserName = _T("");
		strMsg = szMsg;
		cf = cformat;
		time = chattime;
		nTid = 0;
		nMsgSeq = 0;
		nUserCount = userCount;
		nMsgStatus = CMS_INVALID;
		m_nOsType = 0;
	}
	stu_ChatItemInfo& operator = (const stu_ChatItemInfo& ItemInfo )
	{
        if (this == &ItemInfo)
        {
            return *this;
        }
		eMsgGB          = ItemInfo.eMsgGB;	
		strUserID       = ItemInfo.strUserID;
		strNameTalkName = ItemInfo.strNameTalkName; 
		strUserName     = ItemInfo.strUserName; 
		strMsg          = ItemInfo.strMsg;
		cf              = ItemInfo.cf;
		time            = ItemInfo.time;
		nTid            = ItemInfo.nTid;
		nMsgSeq         = ItemInfo.nMsgSeq; 
		nUserCount      = ItemInfo.nUserCount;
		nMsgStatus		= ItemInfo.nMsgStatus;
		m_nOsType		= ItemInfo.m_nOsType;

		return *this;
	}

	
}ChatItemInfo, *LPChatItemInfo;



typedef struct stu_TextRange
{
	int nStart;
	int nEnd;
	stu_TextRange()
	{
		Reset();
	}//lint !e1566

	void Reset()
	{
		nStart = -1;
		nEnd   = -1;
	}

	BOOL IsValid() const
	{
		if (nStart < 0 || nEnd < 0)
		{
			return FALSE;
		}
		return TRUE;
	}

	void Normalize()
	{
		if(nStart > nEnd)
		{
			int nTemp = nStart;
			nStart = nEnd;
			nEnd = nTemp;
		}
	}
}TextRange, *LPTextRange;

// typedef enum tagChatItemState
// {
// 	CHAT_ITEM_S_NONE = 0,
// 	CHAT_ITEM_S_OK
// }CHAT_ITEM_STATE;

class CChatItem
{
public:
	CChatItem(const ChatItemInfo& ChatItemInfo);
	virtual ~CChatItem(void);
	virtual	CString	GetSelectedText() { return TEXT(""); }
	virtual CString GetNameSelectedText();
	virtual CString GetTimeSelectedText(CString strTime);
	virtual void    SelectAll(CString strTime);
	void			SetMsg(LPCTSTR lpsgMsg);

	void			SetToolTip(CString strToolTip);
	CString			GetToolTip();
	WCHAR*			GetFontFaceName();
	FontStyle		GetFontStyle();
	REAL			GetFontHeight();
	CString			GetFailCount();
	virtual BOOL	IsFailedMsg();
	int				IncreaseFailCount();
	void			ResetFailCount()					{ m_nFailCount = 0; }
	void			SetTid(UINT nTid)					{ m_stItemInfo.nTid = nTid; }
	UINT			GetTid()							{ return m_stItemInfo.nTid; }
	void			SetMsgSeq(UINT nMsgSeq)				{ m_stItemInfo.nMsgSeq = nMsgSeq; }
	UINT			GetMsgSeq()							{ return m_stItemInfo.nMsgSeq; }
	virtual void	SetMsgStatus(int nMsgStatus)		{m_stItemInfo.nMsgStatus = nMsgStatus;}
	virtual int		GetMsgStatus()						{return m_stItemInfo.nMsgStatus;}	
	virtual CString GetUserId();

	void            SetItemType(ECHAT_MSG_TYPE eMsgGB)  { m_stItemInfo.eMsgGB = eMsgGB; }
	ECHAT_MSG_TYPE  GetItemType()                       { return m_stItemInfo.eMsgGB; }

	BOOL            IsTalkType();
	BOOL            IsSysType();
	BOOL			IsJoinOrLeaveType();
	BOOL            IsSameUser(LPCTSTR strUserID);
	BOOL            IsNullMsg();
	BOOL			IsGroupUserChat();
	void			SetGroupUserChat(BOOL bVal);
	void			UpdateUserStatus(GTUserStatusType status);
	GTUserStatusType GetUserStatus();
	void			SetUserEmail(CString strEmail);
	CString			GetUserEmail();
	void			SetSucceedCount(int count){m_nSucceedCount = count;}
	int			    GetSucceedCount(){return m_nSucceedCount;}

	LPChatItemInfo  GetChatItemInfo()  { return &m_stItemInfo;}

	LPTextRange GetTimeSelectRange() {return &m_TimeSelectRange;}

	LPTextRange GetNameSelectRange(){return &m_NameSelectRange;}
	virtual CString GetNewMessage();

protected:
	ChatItemInfo    m_stItemInfo;
	TextRange       m_TimeSelectRange;
	TextRange       m_NameSelectRange;
private:
	int				m_nSucceedCount;
	int				m_nFailCount; 

	BOOL			m_bGroupUserChat;
	CString			m_strToolTip;
	GTUserStatusType m_nUserStatus;
	CString			m_strUserEmail;
};

class CChatStickerItem : public CChatItem
{
public:
	CChatStickerItem(const ChatItemInfo& chatItemInfo) : 
	  CChatItem(chatItemInfo), m_bSelected(FALSE) {}
	virtual~CChatStickerItem(){}
	virtual	CString	GetSelectedText();
	virtual void    SelectAll(CString strTime);
	LPCTSTR GetChatStickerID() { return m_stItemInfo.strMsg; }

	virtual CString GetNewMessage();
	BOOL m_bSelected;
};

class CChatUserJoinItem : public CChatItem
{
public:
	CChatUserJoinItem(const ChatItemInfo& ChatItemInfo);

	virtual ~CChatUserJoinItem() {}
	virtual CString GetNewMessage();
	virtual CString GetUserName();
	LPCTSTR GetMsg() { return m_stItemInfo.strMsg; }
	virtual BOOL IsUserJoin();

private:
	BOOL m_bUserJoin;
};
class CChatMsgItem : public CChatItem
{
public:
	CChatMsgItem(const ChatItemInfo& ChatItemInfo);
	virtual~CChatMsgItem();

	virtual	CString	GetSelectedText();
	virtual void    SelectAll(CString strTime);
	LPCTSTR GetMsg() { return m_stItemInfo.strMsg; }
	void    GetUrlVector(std::vector<TextRange>& vcUrl);

	CString GetUrlString(int nIndex);
	LPTextRange GetSelectRange();

	BOOL FindString(LPCTSTR strSearchText);
protected:
	std::vector<TextRange>	m_UrlPosVector;
	TextRange               m_SelectRange;


private:
	void _ParseUrlInMsg(LPCTSTR strMsg);
};


class CChatFileItem : public CChatItem
{
public:
	CChatFileItem(const ChatItemInfo& chatItemInfo) : CChatItem(chatItemInfo) {}
	virtual~CChatFileItem() {}

	void SetTranslateFileInfo(TranslateFileInfoPtr& pTranslateFileInfo) { m_pTranslateFileInfo = pTranslateFileInfo;}
	virtual void    SelectAll(CString strTime);
	TranslateFileInfoPtr GetTranslateFileInfo() {return m_pTranslateFileInfo;}
	BOOL IsFailedMsg();
	virtual CString GetNewMessage();
	void	SetMsgStatus(int nMsgStatus);
	int		GetMsgStatus();
	CString GetUserId();

private:
	TranslateFileInfoPtr m_pTranslateFileInfo;
};